@@ -221,8 +221,8 @@ class WordType(models.Model):
     Count = models.IntegerField(blank=True,null=True)
     Frequency = models.FloatField(blank=True,null=True)
     FreqSource = models.CharField(max_length=100,blank=True,null=True)
-    PhonoProb = models.FloatField(blank=True,null=True)
-    PhonoSource = models.CharField(max_length=100,blank=True,null=True)
+    SPhonoProb = models.FloatField(blank=True,null=True)
+    BiPhonoProb = models.FloatField(blank=True,null=True)
     ND = models.FloatField(blank=True,null=True)
     FWND = models.FloatField(blank=True,null=True)
     CVSkel = models.CharField(max_length=100,blank=True,null=True)
@@ -363,16 +363,7 @@ class WordType(models.Model):
         if self.PhonoProb is None:
             self.setViteLuceProb()
 
-        return self.PhonoProb
-
-    def setPhonoProb(self,source):
-        if source == 'V':
-            self.setViteLuceProb()
-        elif source == 'B':
-            self.PhonoProb = getPhonotacticProb(self.getStrUR(),use_blick=True)
-            self.PhonoSource = 'Blick'
-            self.save()
-
+        return self.SPhonoProb,self.BiPhonoProb
 
     def setViteLuceProb(self):
         SPhoneProb = 0.0
@@ -399,22 +390,9 @@ class WordType(models.Model):
                 for p in allposq:
                     allpos += p.getFreq()
                 BiPhoneProb += float(pos)/float(allpos)
-        self.PhonoProb = (SPhoneProb+BiPhoneProb) / 2.0
-        self.Source = 'ViteLuce'
+        self.SPhonoProb,self.BiPhonoProb = SPhoneProb,BiPhoneProb
         self.save()
 
-#    def getSPhoneProb(self):
-#        if self.SPhoneProb is not None:
-#            return self.SPhoneProb
-#        sp,bp = self.getPhoneProbs()
-#        return sp
-#
-#    def getBiPhoneProb(self):
-#        if self.BiPhoneProb is not None:
-#            return self.BiPhoneProb
-#        sp,bp = self.getPhoneProbs()
-#        return bp
-
 
 class WordToken(models.Model):
     SR = models.ManyToManyField(SegmentType,through=SegmentToken)
@@ -426,8 +404,8 @@ class WordToken(models.Model):
     DialogPart = models.CharField(max_length=1)
     StrVowelF1 = models.FloatField(blank=True,null=True)
     StrVowelF2 = models.FloatField(blank=True,null=True)
-    SemPred = models.FloatField(blank=True,null=True)
-    SynPred = models.FloatField(blank=True,null=True)
+    PrevSemPred = models.FloatField(blank=True,null=True)
+    FollSemPred = models.FloatField(blank=True,null=True)
     NumFormants = models.DecimalField('Number of formants',max_digits=4,decimal_places=1,blank=True,null=True)
     Ceiling = models.IntegerField(blank=True,null=True)
 
@@ -479,9 +457,9 @@ class WordToken(models.Model):
         sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
         return sp
 
-    def getSemContext(self):
-        if self.SemPred is not None:
-            return self.SemPred
+    def getPrevSemPred(self):
+        if self.PrevSemPred is not None:
+            return self.PrevSemPred
         context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getPreviousContext() if lookupCat(str(x)) in ['n','v','a','r'] and str(x) != str(self)]
         cat =lookupCat(str(self))
         if cat == 'NA':
@@ -497,9 +475,31 @@ class WordToken(models.Model):
             else:
                 cat = 'NA'
         sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        self.SemPred = sp
+        self.PrevSemPred = sp
         self.save()
-        return self.SemPred
+        return self.PrevSemPred
+
+    def getFollSemPred(self):
+        if self.FollSemPred is not None:
+            return self.FollSemPred
+        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getFollowingContext() if lookupCat(str(x)) in ['n','v','a','r'] and str(x) != str(self)]
+        cat =lookupCat(str(self))
+        if cat == 'NA':
+            cat = self.Category.Label
+            if 'N' in cat:
+                cat = 'n'
+            elif 'JJ' in cat:
+                cat = 'a'
+            elif 'VB' in cat:
+                cat = 'v'
+            elif 'RB' in cat:
+                cat = 'r'
+            else:
+                cat = 'NA'
+        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
+        self.FollSemPred = sp
+        self.save()
+        return self.FollSemPred
 
     def setStrFormants(self,formants=None):
         if formants is not None:
@@ -607,20 +607,19 @@ class WordToken(models.Model):
                 break
             context.append(prev)
             prev = prev.getPreviousWord()
-            durSum += self.getDuration()
-            if prev is None:
-                break
+            durSum += prev.getDuration()
         return reversed(context)
 
-    def getFollowingContext(self):
-        folls = []
-        f = self.getFollowingWord()
-        for i in range(10):
-            if f is None:
+    def getFollowingContext(self,window=10):
+        context = []
+        foll = self.getFollowingWord()
+        durSum = 0.0
+        while durSum < window:
+            if foll is None:
                 break
-            folls.append(f)
-            f = f.getFollowingWord()
-        return folls
+            context.append(foll)
+            foll = foll.getFollowingWord()
+        return context
 
     def getRecentRepetition(self):
         cont = set(map(str,self.getPreviousContext()))
@@ -844,20 +843,13 @@ class WordToken(models.Model):
             outline['Category'] = self.Category.Label
         elif form['category'] == 'C':
             outline['Category'] = lookupCat(self.WordType.Label)
-        elif form['category'] == 'I':
-            #outline['Category'] = lookupCategory(self.WordType.Label,'IPHOD')
-            pass
         if form['wordDuration']:
             outline['WordDuration'] = self.getDuration()
             outline['BaselineDur'] = self.WordType.getBaseDuration()
         if form['frequency'] == 'B':
             outline['frequency'] = self.WordType.getFreq()
         elif form['frequency'] == 'C':
-            #outline['frequency'] = lookupFreq(self.WordType.Label,'CELEX')
-            pass
-        elif form['frequency'] == 'I':
-            #outline['frequency'] = lookupFreq(self.WordType.Label,'IPHOD')
-            pass
+            outline['frequency'] = lookupFreq(self.WordType.Label,'CELEX')
         if form['gender']:
             outline['SpeakerGender'] = self.Dialog.Speaker.Gender
         if form['age']:
@@ -867,18 +859,22 @@ class WordToken(models.Model):
             outline['OrthoLength'] = len(self.WordType.Label)
         if form['phonoLength']:
             outline['PhonoLength'] = self.WordType.UR.count()
-        if form['nd']:
+        if form['nd'] == 'B':
             outline['NeighDen'],outline['FWND'] = self.WordType.getNDs()
-        if form['phonotactic'] == 'V':
-            if self.WordType.getPhonoSource() != 'ViteLuce':
-                self.WordType.setPhonoProb('V')
+        elif form['nd'] == 'PS':
+            outline['NeighDen'] = getNeighCount(self.WordType.getStrUR(),no_stress=True)
+        elif form['nd'
+        if form['phonotactic'] == 'VB':
+            outline['SPhoneProb'],outline['BiPhoneProb'] = self.WordType.getPhonoProb()
         elif form['phonotactic'] == 'B':
-            if self.WordType.getPhonoSource() != 'Blick':
-                self.WordType.setPhonoProb('B')
-        outline['PhonoProb'] = self.WordType.getPhonoProb()
-        if form['semPred']:
-            outline['SemPred'] = self.getSemContext()
+            outline['BlickPhono'] = getPhonoProb(self.WordType.getStrUR(),use_blick=True,no_stress=True)
+        elif form['phonotactic'] == 'VP':
+            outline['SPhoneProb'],outline['BiPhoneProb'] = getPhonoProb(self.WordType.getStrUR(),use_blick=False,no_stress=True)
+        if form['prev_sem_pred']:
+            outline['PrevSemPred'] = self.getPrevSemPred()
             outline['ImmedSemPred'] = self.getImmediateSemPred()
+        if form['foll_sem_pred']:
+            outline['FollSemPred'] = self.getFollSemPred()
         if form['placeInDialog']:
             outline['placeInDialog'] = self.getDialogPlace()
         outline['AvgSpeakRate'] = self.Dialog.Speaker.getAvgSpeakingRate()
